/*
  $Id: applet_runner.cpp,v 1.1 2006/07/15 08:48:38 noir Exp $
  Copyright (C) 2006 Hiroyuki KUROSAKI <noir (at) st . rim . or . jp>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "applet_runner.h"
#include "rb_funcall2_protect.h"
#include <csignal>
#include <iostream>

using namespace RubyQpeApplet;

AppletRunner::AppletRunner(const QString& applet_file_name,
                           const QString& applet_class_name,
                           const QString& log_file_name,
                           int argc, char *argv[])
    : QPEApplication(argc, argv)
{
    init_ruby_runner(applet_file_name, applet_class_name, log_file_name);
    init_qcop_channel();
}

AppletRunner::~AppletRunner()
{
    finalize_ruby_runner();
}

#include <cstdio>

void sigabrt_handler(int)
{
    int error;
    if (!NIL_P(ruby_errinfo)) {
        printf("Error!\n");
        VALUE args[1];
        args[0] = ruby_errinfo;
        rb_funcall2_protect(rb_cObject,
                            rb_intern("log_error"),
                            1, args, &error);
    }
    else {
        printf("Error, but ruby_errinfo is nil.\n");
    }
}

void AppletRunner::init_ruby_runner(const QString& applet_file_name,
                                    const QString& applet_class_name,
                                    const QString& log_file_name)
{
    ruby_init();
    ruby_init_loadpath();

    char library_name[] = "qpeapplet";

    ruby_script(library_name);

    rb_protect((VALUE (*)(VALUE))rb_require,
               (VALUE)library_name,
               &ruby_state);

    if (ruby_state) {
        abort();
    }
    
    VALUE cRunner = rb_eval_string_protect("Qpe::Applet::Runner",
                                           &ruby_state);

    if (ruby_state) {
        abort();
    }

    VALUE args[3];
    args[0] = rb_str_new2((const char*)applet_file_name.utf8());
    args[1] = rb_str_new2((const char*)applet_class_name.utf8());
    args[2] = rb_str_new2((const char*)log_file_name.utf8());

    ruby_runner = rb_funcall2_protect(cRunner,
                                      rb_intern("new"),
                                      3, args, &ruby_state);
    if (ruby_state) {
        abort();
    }
}

void AppletRunner::run(bool as_main_widget)
{
    
    VALUE applet = rb_funcall2_protect(ruby_runner,
                                       rb_intern("applet"),
                                       0, 0, &ruby_state);
    if (!ruby_state) {
        if (as_main_widget) {
            rb_funcall2_protect(applet,
                                rb_intern("show"),
                                0, 0, &ruby_state);
        }
        else {
            VALUE args[1];
            args[0] = Qfalse;
            rb_funcall2_protect(applet,
                                rb_intern("accept_close="),
                                1, args, &ruby_state);
        }
    }
    exec();
}

void AppletRunner::finalize_ruby_runner()
{
    ruby_runner = Qnil;
    ruby_finalize();
}

void AppletRunner::init_qcop_channel()
{
    qcop_channel = new QCopChannel("QPE/RubyQpeAppletRunner", this);
    connect(qcop_channel,
            SIGNAL(received(const QCString&, const QByteArray&)),
            this,
            SLOT(receive_message(const QCString&, const QByteArray&)));
}

void AppletRunner::receive_message(
    const QCString& msg, const QByteArray&)
{
    if (msg == "redirect_click()") {
        redirect_click();
    }
}

void AppletRunner::redirect_click()
{
    if (!NIL_P(ruby_runner)) {
        VALUE applet = rb_funcall2_protect(ruby_runner,
                                           rb_intern("applet"),
                                           0, 0, &ruby_state);
        if (ruby_state) {
            abort();
        }
        VALUE rsignal = rb_funcall2_protect(applet,
                                            rb_intern("clicked"),
                                            0, 0, &ruby_state);
        if (ruby_state) {
            abort();
        }
        rb_funcall2_protect(rsignal,
                            rb_intern("send"),
                            0, 0, &ruby_state);
        if (ruby_state) {
            abort();
        }
    }
}


int main(int argc, char* argv[])
{
    signal(SIGABRT, sigabrt_handler);

    bool as_main_widget = false;

    if (argc > 1 && QString(argv[1]) == "-m") {
        as_main_widget = true;
        argc--;
        argv++;
    }
    if (argc != 4) {
        std::cerr << "Usage: rqarun [-m] applet_file applet_class log_file" << endl;
        exit(1);
    }
    QString applet_file_name(argv[1]);
    QString applet_class_name(argv[2]);
    QString log_file_name(argv[3]);

    AppletRunner runner(applet_file_name, applet_class_name, log_file_name,
                        argc, argv);

    runner.run(as_main_widget);

    return 0;
}
