# $Id: qpeapplet.rb,v 1.6 2006/07/15 08:48:38 noir Exp $
# Copyright (C) 2006 Hiroyuki KUROSAKI <noir (at) st . rim . or . jp>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require 'qte'
require 'qpe'
require 'qtedecl'

include Qte

module Qpe
  module Applet

    @@log_file = '/dev/null'

    def self.log_file=(new_file)
      @@log_file = new_file
    end

    def self.log_file
      @@log_file
    end

    def self.log_error(errinfo)
      begin
        File.open(@@log_file, 'a') do |f|
          f.puts '---'
          f.puts Time.now
          f.puts errinfo.class.name
          f.puts errinfo.message
          errinfo.backtrace.each do |bt|
            f.puts "\tfrom " + bt
          end
        end
      rescue
        nil
      end
    end

    class AppletError <RuntimeError; end
    # Base class of Ruby Qpe Applet.
    # User must define subclass of this.
    class Base <QWidget
      def initialize(parent = nil, name = '', wflags = 0)
        super
        @accept_close = true
        begin
          user_interface
          @clicked = RSignal.new
          connect(@clicked, self, "toggle_show_hide")
          catchEvent
        rescue
          Qpe::Applet.log_error($!)
        end
      end

      attr_reader :clicked
      attr_accessor :accept_close

      def closeEvent(ev)
        if @accept_close then
          ev.accept
        else
          self.hide
          ev.ignore
        end
      end

      # Define user interface (or anything) in this method.
      def user_interface
        nil
      end

      # default slot for the clicked signal.
      def toggle_show_hide
        begin
          if self.isVisible then
            self.hide
          else
            self.raise
            self.show
          end
        rescue
          Qpe::Applet.log_error($!)
        end
      end
    end

    class NotFoundApplet <Base
      def user_interface
        begin
          self.caption = 'Applet Not Found'
          lb = label('Applet not found.')
          self.fixedSize = lb.sizeHint
        rescue
          Qpe::Applet.log_error($!)
        end
      end
    end

    class Runner <QObject

      def initialize(file, klass, log_file, parent = nil)
        super(parent)
        Qpe::Applet.log_file = log_file
        @applet = nil
        begin
          load(file)
          @klass = klass.gsub(/[\`{}#]+/, '')
          @applet = eval("#{@klass}.new")
        rescue
          log_error($!)
        end
      end

      attr_reader :applet

      # not used by rqarun
      def run(as_main_widget = false)
        begin
          a = Qpe::QPEApplication.new(['rqarun'])
          if as_main_widget then
            a.setMainWidget(@applet)
            @applet.show
          end
          a.exec
        rescue
          log_error($!)
        end
      end

    end

  end
end

def log_error(errinfo)
  Qpe::Applet.log_error(errinfo)
end
